@@ -1,4 +1,4 @@
-from typing import Protocol
+from typing import List, Protocol
 
 from pyspark.sql import DataFrame
 
@@ -29,6 +29,14 @@ class BaseServer(Protocol):
     ):
         pass
 
+    def upsert_to_table_by_name(
+        self,
+        df_source: DataFrame,
+        table_name: str,
+        join_cols: List[str],
+    ):
+        pass
+
     def truncate_table_by_name(self, table_name: str) -> None:
         pass
 
