@@ -1,8 +1,10 @@
+import contextlib
 import importlib.resources
 import re
 import time
+import uuid
 from types import ModuleType
-from typing import Union
+from typing import List, Union
 
 import pyodbc
 from pyspark.sql import DataFrame
@@ -10,6 +12,7 @@ from pyspark.sql import DataFrame
 from atc.config_master import TableConfigurator
 from atc.spark import Spark
 from atc.sql.sql_handle import SqlHandle
+from atc.utils import GetMergeStatement
 
 
 class SqlServer:
@@ -85,12 +88,20 @@ class SqlServer:
 
         return hostname, port, username, password, database
 
-    def execute_sql(self, sql: str):
-        self.test_odbc_connection()
+    @contextlib.contextmanager
+    def connect_to_db(self):
         conn = pyodbc.connect(self.odbc)
         conn.autocommit = True
-        conn.execute(sql)
-        conn.close()
+        try:
+            yield conn
+        finally:
+            conn.close()
+
+    def execute_sql(self, sql: str):
+        self.test_odbc_connection()
+
+        with self.connect_to_db() as conn:
+            conn.execute(sql)
 
     def from_tc(self, id: str) -> SqlHandle:
         """This method allows an instance of SqlServer to be a drop in for the class
@@ -107,7 +118,9 @@ class SqlServer:
     def load_sql(self, sql: str):
         self.test_odbc_connection()
         return Spark.get().read.jdbc(
-            url=self.url, table=sql, properties=self.properties
+            url=self.url,
+            table=sql,
+            properties=self.properties,
         )
 
     def read_table_by_name(self, table_name: str):
@@ -140,6 +153,76 @@ class SqlServer:
             "password", self.password
         ).save()
 
+    def upsert_to_table_by_name(
+        self,
+        df_source: DataFrame,
+        table_name: str,
+        join_cols: List[str],
+        filter_join_cols: bool = True,
+        overwrite_if_target_is_empty: bool = True,
+        big_data_set: bool = True,
+        batch_size: int = 10 * 1024,
+        partition_count: int = 60,
+    ):
+
+        if df_source is None:
+            return None
+
+        if filter_join_cols:
+            df_source = df_source.filter(
+                " AND ".join(f"({col} is NOT NULL)" for col in join_cols)
+            )
+            print(
+                "Rows with NULL join keys found in input dataframe"
+                " will be discarded before load."
+            )
+
+        if overwrite_if_target_is_empty:
+            df_target = self.read_table_by_name(table_name=table_name)
+
+            if len(df_target.take(1)) == 0:
+                return self.write_table_by_name(
+                    df_source=df_source,
+                    table_name=table_name,
+                    append=False,
+                    big_data_set=big_data_set,
+                    batch_size=batch_size,
+                    partition_count=partition_count,
+                )
+
+        # Define name of temp stagning table
+        # ## defines the table as a temp sql table
+        staging_table_name = f"##{uuid.uuid4().hex}"
+
+        with self.connect_to_db() as conn:
+            # Create temp staging table for merge based source table
+            conn.execute(
+                f"""
+                SELECT * INTO {staging_table_name}
+                FROM {table_name} WHERE 1 = 0;
+                """
+            )
+
+            self.write_table_by_name(
+                df_source=df_source,
+                table_name=staging_table_name,
+                append=False,
+                big_data_set=big_data_set,
+                batch_size=batch_size,
+                partition_count=partition_count,
+            )
+
+            mergeQuery = GetMergeStatement(
+                merge_statement_type="sql",
+                target_table_name=table_name,
+                source_table_name=staging_table_name,
+                join_cols=join_cols,
+                insert_cols=df_source.columns,
+                update_cols=df_source.columns,
+            )
+
+            conn.execute(mergeQuery)
+
     def truncate_table_by_name(self, table_name: str):
         self.execute_sql(f"TRUNCATE TABLE {table_name}")
 
