@@ -1,4 +1,5 @@
 from abc import ABC
+from typing import List, Union
 
 from pyspark.sql import DataFrame
 
@@ -21,3 +22,12 @@ class TableHandle(ABC):
 
     def drop_and_delete(self) -> None:
         pass
+
+    def write_or_append(self, df: DataFrame, mode: str) -> None:
+        pass
+
+    def upsert(self, df: DataFrame, join_cols: List[str]) -> Union[DataFrame, None]:
+        pass
+
+    def get_tablename(self) -> str:
+        pass
